#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2002-2003 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: HtmlBrowser.py,v 1.10 2004/01/21 22:40:32 vaclavslavik Exp $
#
#  Abstract interface to various browsers (Mozilla, MSIE, wxHTML)
#

from wxPython import *
from wxPython.wx import *
from wxPython.html import *
import sys, os, os.path
import utils

HtmlBrowser = None

# -------------------------------------------------------------------------
#                                Mozilla
# -------------------------------------------------------------------------

try:
    from wxPython.mozilla import *

    try:
        import wxPython.mozilla
        MOZILLA_VERSION = wxPython.mozilla.__version__
    except AttributeError:
        raise RuntimeError('Documancer needs wxMozilla >= 0.5.3')
    
    if sys.platform == 'win32':
        wxMozillaSettings_SetMozillaPath('%s\\mozilla' %
                                         os.environ['DOCUMANCER_HOME'])
   
    wxMozillaSettings_SetProfilePath(
                    os.path.join(utils.getConfigDir(), 'mozilla-profile'))
    
    wxMozillaSettings_SetIntPref("browser.downloadmanager.behavior", 1)

    class MozillaBrowser(wxMozillaBrowser):
        def __init__(self, parent, id):
            wxMozillaBrowser.__init__(self, parent, id)
            self.OnPageChanged = lambda x: x
            self.OnStatusChanged = lambda x,y: x
            self.OnTitleChanged = lambda x: x
            self.OnProgress = lambda x: x
            self.url = ''
            self.name = 'Mozilla'
            EVT_MOZILLA_URL_CHANGED(self, -1, self.__OnUrlChanged)
            EVT_MOZILLA_STATUS_CHANGED(self, -1, self.__OnStatusChanged)
            EVT_MOZILLA_TITLE_CHANGED(self, -1, self.__OnTitleChanged)
            EVT_MOZILLA_PROGRESS(self, -1, self.__OnProgress)

        def __OnUrlChanged(self, event):
            self.url = event.GetNewURL()
            self.OnPageChanged(self.url)

        def __OnStatusChanged(self, event):
            self.OnStatusChanged(event.GetStatusText(), event.IsBusy())
        
        def __OnTitleChanged(self, event):
            self.OnTitleChanged(event.GetTitle())
        
        def __OnProgress(self, event):
            if event.GetTotalMaxProgress() != -1:
                self.OnProgress(float(event.GetTotalCurrentProgress()) /
                                float(event.GetTotalMaxProgress()))
            elif event.GetSelfMaxProgress() != -1:
                self.OnProgress(float(event.GetSelfCurrentProgress()) /
                                float(event.GetSelfMaxProgress()))
        
        def GetPageTitle(self):
            return self.GetTitle()
        def GetPageURL(self):
            return self.url

        def SavePrefs(self):
            wxMozillaSettings_SavePrefs()

    HtmlBrowser = MozillaBrowser

except ImportError:
    pass


# -------------------------------------------------------------------------
#                                wxIEHtmlWin
# -------------------------------------------------------------------------

if HtmlBrowser == None: # we prefer Mozilla

    try:
        from wxPython.lib.activexwrapper import MakeActiveXClass
        import HtmlBrowser_activex_ie
        WebBrowserClass = MakeActiveXClass(HtmlBrowser_activex_ie.WebBrowser)
    
        class MsieBrowser(WebBrowserClass):
            def __init__(self, parent, id):
                WebBrowserClass.__init__(self, parent, id,
                                         style=wxNO_FULL_REPAINT_ON_RESIZE)
                self.OnPageChanged = lambda x: x
                self.name = 'Internet Explorer'
        
            def OnNavigateComplete2(self, pDisp, URL):
                self.OnPageChanged(URL)

            def LoadURL(self, url):
                self.Navigate(url)
            def GetPageTitle(self):
                if self.LocationName == self.LocationURL:
                    return ''
                else:
                    return self.LocationName
            def GetPageURL(self):
                return self.LocationURL
            def CanGoBack(self):
                return 1
            def CanGoForward(self):
                return 1

        HtmlBrowser = MsieBrowser
        
    except ImportError:
        pass



# -------------------------------------------------------------------------
#                           wxHtmlWindow (fallback)
# -------------------------------------------------------------------------

if HtmlBrowser == None:
    
    print 'warning: using minimalistic HTML renderer (wxHTML),'
    print '         please install wxMozilla!'
    
    class wxHtmlBrowser(wxHtmlWindow):
        def __init__(self, parent, id):
            wxHtmlWindow.__init__(self, parent, id)
            self.OnPageChanged = lambda x: x
            self.name = 'wxHTML'

        def LoadURL(self, url):
            self.LoadPage(url)
        def GoBack(self):
            self.HistoryBack()
        def GoForward(self):
            self.HistoryForward()
        def GetPageTitle(self):
            return self.GetOpenedPageTitle()
        def GetPageURL(self):
            return self.GetOpenedPage()
        def CanGoBack(self):
            return self.HistoryCanBack()
        def CanGoForward(self):
            return self.HistoryCanForward()

    HtmlBrowser = wxHtmlBrowser
