#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2003 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: app.py,v 1.4 2003/12/12 21:42:55 vaclavslavik Exp $
#
#  wxPython application
#

from wxPython.wx import *
from MainFrame import MainFrame
import BusyIndicator
import utils, book, server

class UICallback_wx:
    def __init__(self, mainFrame):
        self.mainFrame = mainFrame
        self.busyCnt = 0
        self.busy = BusyIndicator.BusyIndicator(self.mainFrame)
    def error(self, msg):
        wxLogError(msg)
    def showBusyIndicator(self):
        if self.busyCnt == 0:
            self.busy.show()
        self.busyCnt += 1
    def hideBusyIndicator(self):
        self.busyCnt -= 1
        if self.busyCnt == 0:
            self.busy.hide()
    def setBusyText(self, text):
        self.busy.setText(text)
    def busyUpdate(self):
        self.busy.update()


class DocumancerApp(wxApp):
    def OnInit(self):
        self.instanceChecker = \
            wxSingleInstanceChecker('documancer-%s' % wxGetUserId(),
                                    utils.getConfigDir())
        if self.instanceChecker.IsAnotherRunning():
            wxMessageBox("""\
Another Documancer instance is already running, exiting.""" ,
                         "Documancer Error", wxOK | wxICON_ERROR)
            return 0
        
        book.loadBooks()
        server.runServer(inBackground=1)
        
        wxInitAllImageHandlers()
        main_frame = MainFrame(None, -1, "", name='Documancer')
        utils.uiCallback = UICallback_wx(main_frame)
        self.SetTopWindow(main_frame)
        main_frame.Show(1)
        main_frame.SetSize(
             (utils.config().ReadInt('/GuiState/MainFrame/width',500),
              utils.config().ReadInt('/GuiState/MainFrame/height',400)))
        return 1

    def OnExit(self):
        book.saveBooks()
        self.instanceChecker = None

# end of class DocumancerApp

def run():
    app = DocumancerApp(0)
    app.MainLoop()

if __name__ == '__main__':
    run()
