#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2002,2003,2004 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: indexer.py,v 1.9 2004/01/10 21:08:20 vaclavslavik Exp $
#
#  Fulltext search index, using SWISH-e
#


import utils, os, os.path, tempfile, urllib, sys, re, string, shutil
import providers, book


def getIndexDirName(book):
    s = utils.mangleBookName(book.title)
    idir = utils.getIndexesDir() + s
    return idir

def enableIndex(book):
    idir = getIndexDirName(book)
    if not os.path.isdir(idir): os.mkdir(idir)    

def disableIndex(book):
    idir = getIndexDirName(book)
    os.system("rm -rf %s" % idir)

def updateIndex(book):
    utils.uiCallback.showBusyIndicator()
    utils.uiCallback.setBusyText("Updating index: %s" % book.title)

    idir = getIndexDirName(book)
    port = utils.getServerPort()
    temp = tempfile.mktemp()
    os.mkdir(temp)
  
    configFile = tempfile.mktemp()
    cfgF = open(configFile, 'wt')
    safetemp = temp
    if sys.platform == 'win32':
        safetemp = safetemp.replace('\\','/')
    safetemp = safetemp.replace(".", "\\.").replace("@", "\\@")
    wordchars = string.digits + string.letters + '_~'
    cfgF.write("""\
UseStemming yes
ReplaceRules remove "^%(safetemp)s"
ReplaceRules replace " " "%%20"
WordCharacters %(words)s
BeginCharacters %(words)s
EndCharacters %(words)s
""" % {'safetemp':safetemp,'words':wordchars})
    cfgF.close()
    
    prov = book.getProviderObj()
    startURL = prov.getURLForIndexing(book)

    def filter_prg(line):
        utils.uiCallback.busyUpdate()
        return

    utils.uiCallback.setBusyText("Updating index: %s (fetching...)" % book.title)
    # First, use wget to download entire book to a temporary file:
    # (NB: we need to change directory to out temp directory because some
    #      versions of wget don't like it when the path passed to -P flag
    #      contains @, which happens to be the case with Python-generated
    #      temp filenames)
    #      
    # -np   no parent
    # -nv   non verbose
    # -P    directory prefix
    # -r    recursive
    # -nH   no host-prefixed directories
    if sys.platform == 'win32':
        cwd = os.getcwd()
        os.chdir(temp)
        cmd =  "wget -nv -np -r -nH http://localhost:%i/%s/%s" % \
                (port, utils.mangleBookName(book.title), startURL)
        utils.runCommandWithFilter(cmd, filter_prg)
        os.chdir(cwd)
    else:
        cmd =  "cd %s ; wget -nv -np -r -nH http://localhost:%i/%s/%s 2>&1" % \
               (temp, port, utils.mangleBookName(book.title), startURL)
        utils.runCommandWithFilter(cmd, filter_prg)

    utils.uiCallback.setBusyText("Updating index: %s (indexing...)" % book.title)
    # Use swish-e to create the index:
    utils.runCommandWithFilter(
          'swish-e -f "%s/index.swish-e" -c %s -i %s' % 
           (idir, configFile, temp), filter_prg)
    
    # Remove downloaded junk
    shutil.rmtree(temp, ignore_errors=1)
    os.remove(configFile)

    utils.uiCallback.hideBusyIndicator()

def __ensureBookIndexValidity(b):
    if b.getAttr(book.ATTR_DIRTY) == '1':
        if b.getAttr(book.ATTR_INDEXED) == '1':
            enableIndex(b)
            updateIndex(b)
        else:
            disableIndex(b)
        b.setAttr(book.ATTR_DIRTY, '0')


httpUrlPrefixRE = re.compile(r"(/[^ ]*)")
httpUrlRepl = None

def search(b, query):
    __ensureBookIndexValidity(b)
    if b.getAttr(book.ATTR_INDEXED) != '1':
        utils.uiCallback.error(
                "Book '%s' doesn't have fulltext index!" % b.title)
        return []
    
    global httpUrlPrefix, httpUrlRepl
    idir = getIndexDirName(b)

    httpUrlRepl = (
        "http://localhost:%i\\1?query=%s" %
            (utils.getServerPort(), urllib.quote(query, "")))

    result = []
    
    def filter_swish(line):
        global httpUrlPrefixRE, httpUrlRepl
        if len(line) == 0 or line[0] == '#' or line[:4] == 'err:' or line == '.\n':
            return
        else:
            spl = line.strip().split('\t')
            score = spl[0]
            url = httpUrlPrefixRE.sub(httpUrlRepl, spl[1])
            title = spl[2]
            result.append((title,url,score))

    # Retrieve matches from the index:
    utils.runCommandWithFilter(
                'swish-e -f "%s/index.swish-e" -w %s -d "\t"' %
                (str(idir), str(query)), filter_swish)
    return result
