#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2002 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: __init__.py,v 1.6 2003/09/18 22:47:29 vaclavslavik Exp $
#
#  Interface that providers must implement
#

import sys

# Provider interface:
class DocumentationProvider:
    def getID(self):
        """Returns human-readable string identifying the provider"""
        return

    def getDescription(self):
        """Returns human-readable description of the provider."""
        return ''

    def serve(self, book, url):
        """Returns tuple (data_string,mime_type_string)"""
        return

    def getURLForIndexing(self, book):
        """Returns URL where the indexer should start"""
        return

    def getHomeURL(self, book):
        """Return URL where the browser should go if "Home" is pressed."""
        return

    def getConfigurationDialog(self, parent):
        """Returns wxPython wxPanel object that can be used to configure
           the provider. This wxPanel must have 
             - SetData function (accepting book.attr dictionary)
             - GetData that return the dict
           It must call UpdateBook method of object given as argument to
           its SetDialog method after any change)."""
        return None

providers = {}

def register(provider):
    global providers
    providers[provider.getID()] = provider


# import the providers:

import html.provider
import pythondoc.provider
if sys.platform != 'win32':
    import man.provider
    import info.provider
