#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2002-2003 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: provider.py,v 1.3 2003/09/18 22:47:21 vaclavslavik Exp $
#
#  Manual pages documentation provider (Unix)
#


import providers
import os.path, commands, string, urllib, re


# where to find manpages:
manPaths = None
# cached indexes:
manpagesLists = {}

def getManpagesList(section, filter):
    global manPaths, manpagesLists
    if section in manpagesLists:
        manlist = manpagesLists[section]
    else:
        if manPaths == None:
            manPaths = commands.getoutput('man -w').split(':')

        manlist = []
        for path in manPaths:
            searchPath = '%s/man%s' % (path, section)
            if not os.path.isdir(searchPath): continue
            for file in os.listdir(searchPath):
                _file = '%s/%s' % (searchPath, file)
                if os.path.islink(_file):
                    filename = os.path.basename(os.readlink(_file))
                else:
                    filename = file
                manfile = urllib.quote(filename[:filename.find('.')])
                manname = file[:file.find('.')]
                manlist.append((manname.lower(),manfile,manname))
        manlist.sort()
        manpagesLists[section] = manlist
        
    if filter == '':
        return manlist
    else:
        filterRegex = re.compile(filter)   
        return [x for x in manlist if filterRegex.match(x[2])]
    

def generateIndex(section, filter):
    manlist = getManpagesList(section, filter)
    out_abc = ''
    out_list = ''
    letter = '?'
    for manid,manfile,manname in manlist:
        if len(manname) == 0: continue
        if manid[0] != letter:
            letter = manid[0]
            l = string.upper(letter)
            out_abc += ' <a href="#%s">%s</a> ' % (l,l)
            out_list += '<h4><a name="%s">%s</h4>' % (l,l)
        out_list += '<a href="./%s.html">%s</a><br>\n' % (manfile,manname)

    output = """
<html>
<title>Section %s</title>
<link href="man2html.css" rel="stylesheet" type="text/css" />
<body>
<h1>Manual Pages Section %s</h1>
<p>
%s
<p>
%s
</body></html>
""" % (section,section, out_abc, out_list)

    return output


class ManProvider(providers.DocumentationProvider):

    def getID(self):
        return "Man pages"

    def serve(self, book, url):
        # Stylesheet:
        if url[-12:] == 'man2html.css':
            f = open('%s/helpers/man2html/man2html.css' % os.getenv('DOCUMANCER_HOME'), 'rt')
            data = f.read()
            f.close()
            return (data, 'text/css')
    
        if url[-5:] == ".html": url = url[:-5]     
        if url == '':
            url = 'index'
        urls = url.split("/")
        
        # Global index:
        if url == 'index':
            f = open('%s/helpers/man2html/index.html' % os.getenv('DOCUMANCER_HOME'), 'rt')
            data = f.read()
            f.close()
            return (data, 'text/html')

        # Section indexes:
        elif len(urls) == 2 and urls[1] == '':            
            return (generateIndex(urls[0], book.getAttr('regex')), 'text/html')

        # Man pages:
        elif len(urls) == 2:
            section=urls[0]
            page=urls[1]
            interim=commands.getoutput("helpers/man2html/man2html.cgi %s %s" % (section,page))
            # NB: man2html has annoying misfeature of adding \007 (bell)
            #     characters around section name, kill them:
            return (interim.replace('\007',''), 'text/html')

        else:
            return None

    def getHomeURL(self, book):
        s = book.getAttr('section')
        if s == '':
            return 'index.html'
        else:
            return '%s/' % s

    def getURLForIndexing(self, book):
        return self.getHomeURL(book)

    def getConfigurationDialog(self, parent):
        from confgui import ManSettingsPanel
        return ManSettingsPanel(parent, -1)


providers.register(ManProvider())
